// lxcf.cc -- compute the local cross correlation function.

// To use this file, your version of Octave must support dynamic
// linking.  To find out if it does, type the command
//
//   x = octave_config_info; x.DEFS
//
// at the Octave prompt.  Support for dynamic linking is included if
// the output contains the string -DWITH_DYNAMIC_LINKING=1.
//
// To compile this file, type the command
//
// mkoctfile --verbose lxcf.cc
//
// at the shell prompt.  The script mkoctfile should have been
// installed along with Octave.  Running it will create a file called
// lxcf.oct that can be loaded by Octave.
//
// Enter:
//
// help lxcf
//
// at the Octave prompt to have a description of how lxcf should be used.  
// 
// This file may be used under the terms defined by the GNU 
// General Public License [write to the Free Software Foundation, 
// 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA]
//
// WITHOUT ANY WARRANTY; without even the implied warranty 
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
//
// Author: Eric Chassande-Mottin [ecm@obs-nice.fr], Sun Feb 22 12:42:23 GMT 2004

//

#include <octave/config.h>

#include <iostream>
#include <sstream>
using namespace std;
#include <string>

#include <octave/defun-dld.h>
#include <octave/error.h>
#include <octave/oct-obj.h>
#include <octave/pager.h>
#include <octave/symtab.h>
#include <octave/variables.h>

static inline int
getmin ( int x, int y)
{
  return x < y ? x : y;
}

DEFUN_DLD (lxcf, args, ,
  "LXCF: compute the local cross correlation function.\n")
{
  octave_value_list retval;
  
  /*----------- get input parameters ----------------*/
  int nargin = args.length ();
  
  if (nargin < 3)
    {
      // print_usage("lxcf");
      error("Usage: out = lxcf(in1,in2,t)");
      return retval;
    }

  /* get the data */
  ComplexColumnVector in1(args(0).complex_vector_value());
  ComplexColumnVector in2(args(1).complex_vector_value());
  if (in1.length()!=in2.length())
    {
      error("lxcf: in1 and in2 must have same length");
      return retval;
    }

  /* get the time vector */
  ColumnVector t(args(2).vector_value());
  
  /* create output data */
  int N=in1.length();
  int tcol=t.length();
  ComplexMatrix out(N,tcol,0);

  /* compute lxcf */
  int taumax;
  for (int n=0 ; n<tcol ; n++)
    {    
      int ti= (int) t(n)-1; 
      taumax=getmin(ti,N-1-ti);
      for (int tau=-taumax; tau<=taumax ; tau++)
	out((N+tau)%N,n)=in1(ti+tau)*conj(in2(ti-tau));
    }
  
  retval(0)=out;
  
  return retval;
}





